@@ -56,18 +56,21 @@ pub trait LiteTask: Sized + Send + 'static {
         loop {
             let last_attempt = Instant::now();
             let routine_result = self.repeatable_routine().await;
-            match routine_result {
-                Ok(Some(output)) => {
-                    return Ok(output);
+            let instant = {
+                match routine_result {
+                    Ok(Some(output)) => {
+                        break Ok(output);
+                    }
+                    Ok(None) => {
+                        // Continue
+                        self.retry_at(last_attempt, true)
+                    }
+                    Err(err) => {
+                        log::error!("Routine {} failed: {}", self.name(), err);
+                        self.retry_at(last_attempt, false)
+                    }
                 }
-                Ok(None) => {
-                    // continue
-                }
-                Err(err) => {
-                    log::error!("Routine {} failed: {}", self.name(), err);
-                }
-            }
-            let instant = self.retry_at(last_attempt);
+            };
             crate::compat::delay_until(instant).await;
         }
     }
@@ -80,12 +83,16 @@ pub trait LiteTask: Sized + Send + 'static {
     }
 
     /// When to do the next attempt for `repeatable_routine`.
-    fn retry_at(&self, _last_attempt: Instant) -> Instant {
-        Instant::now() + self.retry_delay(_last_attempt)
+    ///
+    /// `succeed` means the last attempt was successful.
+    fn retry_at(&self, _last_attempt: Instant, succeed: bool) -> Instant {
+        Instant::now() + self.retry_delay(_last_attempt, succeed)
     }
 
     /// How long to wait to retry. Called by `retry_at` method.
-    fn retry_delay(&self, _last_attempt: Instant) -> Duration {
+    ///
+    /// `succeed` means the last attempt was successful.
+    fn retry_delay(&self, _last_attempt: Instant, _succeed: bool) -> Duration {
         Duration::from_secs(5)
     }
 }
